/*
 * ext3rminator - a program to recover deleted data from an ext3 partition
 * Copyright (c) 2002-2003 Mike Hommey <mh@glandium.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _EXT3RMINATOR_H_
#define _EXT3RMINATOR_H

#define ENABLE_SWAPFS

#include <ext2fs/ext2fs.h>

#define BLOCK_EMPTY            0x80
#define BLOCK_VALID            0x20
#define BLOCK_INDIRECT         0x18
#define BLOCK_MULTIPLE_POINTED 0x04
#define BLOCK_POINTED          0x02
#define BLOCK_USED             0x01

#define BLOCK_DOUBLE_INDIRECT  0x08
#define BLOCK_TRIPLE_INDIRECT  0x10

#define BLOCK_FREE             0

#define BLOCK_SET_EXACT(block_map, block_num, flags) ((block_map)[block_num] = (flags))
#define BLOCK_SET(block_map, block_num, flags) ((block_map)[block_num] |= (flags))
#define BLOCK_UNSET(block_map, block_num, flags) ((block_map)[block_num] &= ~(flags))
#define BLOCK_IS_SET(block_map, block_num, flags) ((block_map)[block_num] & (flags))
#define BLOCK_IS_SET_ALL(block_map, block_num, flags) (((block_map)[block_num] & (flags)) == flags)
#define BLOCK_IS(block_map, block_num, flags) ((block_map)[block_num] == (flags))
#define BLOCK_IS_FREE(block_map, block_num) ((block_map)[block_num] == 0)

struct ext3rminator {
  int diskmap_fd;
  char *device, *diskmap, *destdir;
  int newdiskmap, device_is_file, free_blocks;

  ext2_filsys fs;
  unsigned char *block_map;
};

typedef struct ext3rminator ext3rminator;

ext3rminator *ext3r_create();

int ext3r_set_device(ext3rminator *ext3rminator, char *filename);

int ext3r_set_diskmap(ext3rminator *ext3rminator, char *filename);

int ext3r_set_destdir(ext3rminator *ext3rminator, char *filename);

int ext3r_init_device(ext3rminator *ext3rminator);

int ext3r_init_diskmap(ext3rminator *ext3rminator);

int ext3r_save_diskmap(ext3rminator *ext3rminator);

int ext3r_explore(ext3rminator *ext3rminator);

int ext3r_extract(ext3rminator *ext3rminator);

int ext3r_destroy(ext3rminator *ext3rminator);

#endif /* _EXT3RMINATOR_H_ */
