/*
 * ext3rminator - a program to recover deleted data from an ext3 partition
 * Copyright (c) 2002-2003 Mike Hommey <mh@glandium.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#define _GNU_SOURCE

#include <getopt.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <config.h>
#include "ext3rminator.h"

#define VERSION_STRING "ext3rminator version " VERSION

void version(void) {
  printf(VERSION_STRING ".\n");
}

void usage(void) {
  version();
  printf("\nUsage: ext3rminator -d device [-m diskmap] [-r destdir] [-h] [-v]\n" \
	 "Tries to recover deleted data from an ext3 filesystem.\n" \
	 "\n" \
         "  -h, --help      Show this help message.\n" \
	 "  -v, --version   Show version of program.\n" \
	 "  -d device       Device (or file) containing the ext3 filesystem.\n" \
         "  -m diskmap      File in which intermediate internal data for ext3rminator\n" \
	 "                  may be stored.\n" \
	 "  -r destdir      Directory where to put recovered files.\n");
}

int main(int argc, char *argv[]) {
  int c, option_index = 0;
  ext3rminator *ext3rminator = NULL;
  static struct option long_options[] = {
    { "help", 0, 0, 'h' },
    { "version", 0, 0, 'v' },
    { "device", 1, 0, 'd' },
    { "map", 1, 0, 'm' },
    { "destdir", 1, 0, 'r' },
    { 0, 0, 0, 0 }
  };

  if (!(ext3rminator = ext3r_create())) {
    return 1;
  }

  while (1) {
    c = getopt_long(argc, argv, "d:m:r:hv", long_options, &option_index);
    if (c == -1)
      break;
    switch (c) {
    case 'd':
      if (!ext3r_set_device(ext3rminator, optarg))
	goto error;
      break;
    case 'm':
      if (!ext3r_set_diskmap(ext3rminator, optarg))
	goto error;
      break;
    case 'r':
      if (!ext3r_set_destdir(ext3rminator, optarg))
	goto error;
      break;
    case '?':
    case 'h':
      usage();
      return 1;
    case 'v':
      version();
      return 1;
    default:
      printf("GetOpt sould NOT return %c !\n", c);
      return 1;
    }
  }
  if (!ext3rminator->device) {
    usage();
    goto error;
  }

  printf(VERSION_STRING ".\n");

  if (!ext3r_init_device(ext3rminator))
    goto error;

  if (!ext3r_init_diskmap(ext3rminator))
    goto error;

  if (!ext3r_explore(ext3rminator))
    goto error;

  if (!ext3r_extract(ext3rminator))
    goto error;

  ext3r_destroy(ext3rminator);
  return 0;

 error:
  ext3r_destroy(ext3rminator);
  return 1;
}
